@@ -27,8 +27,10 @@
 #include "playlist-treeview.h"
 
 gboolean
-init_ui                         (gint             *argc,
-                                 gchar           **argv[]);
+init_ui                         (void);
+
+GtkButton *
+get_rescan_button               (void);
 
 void
 deinit_ui                       (void);
