@@ -18,11 +18,11 @@
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#include <config.h>
+
 #include <string.h>
 #include <stdlib.h>
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+
 #include <gmodule.h>
 
 #include "playlist-treeview.h"
@@ -1024,7 +1024,6 @@ remove_media_server (GUPnPDeviceProxy *proxy)
                       compare_media_server,
                       (gpointer) udn,
                       FALSE)) {
-                unschedule_icon_update (info);
                 gtk_tree_store_remove (GTK_TREE_STORE (model), &iter);
         }
 }
