@@ -20,13 +20,11 @@
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#include <config.h>
+
 #include <string.h>
 #include <stdlib.h>
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <gmodule.h>
 #include <glib/gi18n.h>
 
@@ -421,8 +419,10 @@ get_action_arg_widget_info (GtkWidget                 *arg_widget,
                                         introspection,
                                         arg_info->related_state_variable);
 
-        if (arg_info->direction == GUPNP_SERVICE_ACTION_ARG_DIRECTION_OUT)
+        if (arg_info->direction == GUPNP_SERVICE_ACTION_ARG_DIRECTION_OUT) {
+                *value = NULL;
                 return arg_info->name;
+        }
 
         *value = g_slice_alloc0 (sizeof (GValue));
         g_value_init (*value, variable_info->type);
